% Wrapper function for mex
% 'csmex' will find the csCore folder on path and add this folder
% as the include and library folder option to mex
% The CyberSeaMsgDll.lib library is added to the link command
%
% Copyright:	Marine Cybernetics AS, Trondheim, Norway
% Author:		Nicolai Husteli
% Date:			2007-09-21
%
% Revisions:    2007-09-28 NH: Found wrong folder if there were more than
%                              one subfolder under \csCore. This is fixed.
%               2008-04-01 NH: Bugzilla ID 252. Made it possible to enter 
%                              mex command-line options as second argument '
%                              to the function. csmex('winRTclock.c','-g')
%                              
%
%
% Version number: $Revision: 1 $


function csmex(argument,option)

if nargin == 1    
    cmd   = '-v';
elseif nargin == 2    
    cmd   = option;
end

try
csCoreFolder = getCsCoreSfuncFolderOnPath;
if isempty(csCoreFolder)
    disp('\csCore\sfunc folder not found. Check that CyberSea Architecture is added to path');
    return;
else
    disp(['Found this \csCore\sfunc folder: ' csCoreFolder]);
end
mexcmd = ['mex ' cmd ' ' argument ' -I''' csCoreFolder '''' ' -L''' csCoreFolder '''' ' -lCyberSeaMsgDLL']
eval(mexcmd);
disp('Sucessfull compilation');
catch   
    disp('Error compiling sfunction.');
end



%% Local functions 
% Find \csCore\sfunc folder based on path
function [csCoreFolder] = getCsCoreSfuncFolderOnPath()

mp = matlabpath;
csCoreFolder = '';
% Split path string based on the character ';'
[start_idx, end_idx, extents, matches, tokens, names, splits] = regexp(mp, ';');
for i=1:length(splits) 
    if ~isempty(strfind(splits{i},'csCore\sfunc')) 
        csCoreFolder = splits{i}
    end        
end
