function manager(modelname, solver, stepsize)
    close all
    clc
    % MYGUI Brief description of GUI.
    %       Comments displayed at the command line in response 
    %       to the help command. 

%%  Initialization tasks
    global ParT simstatus h_timer;
    simstatus = 'initialise';
    
    open_system(modelname);

    set_param(modelname, 'Solver', solver);
    if strcmp(solver, 'ode3')
        set_param(modelname, 'FixedStep', stepsize);
    end

    %%  Construct the components
    % Create the main GUI window
    figPos = [50, 80, 500, 550];
    % [0, figPos(4)-160, figPos(3), 120]
    h = figure('Name', 'Crane Simulator', 'tag', 'fig:main', 'Position', figPos, 'NumberTitle', 'off', 'Toolbar', 'none', 'Menubar', 'none');

    panTopPos = [0 figPos(4)-30 figPos(3) 30];
    % Top panel
    h1 = uipanel('Parent', h, 'tag','pan:sim', 'Units', 'pixels', 'Position', panTopPos); % , 'Title', 'Simulator controls'
    button.width = 50;
    button.height = 20;
    button.left = 10;
    button.bottom = 5;
    button.hnext = button.width + 5;
    h2 = uicontrol(h1, 'Style', 'PushButton', 'String', 'Start', 'tag', 'but:start', 'Position', [button.left button.bottom button.width button.height], 'Callback', {@cb_startpause,modelname, stepsize});
    button.left = button.left + button.hnext;
    uicontrol(h1, 'Style', 'PushButton', 'String', 'Reset', 'tag', 'but:reset', 'Position', [button.left button.bottom button.width button.height])
    button.left = button.left + button.hnext;
    uicontrol(h1, 'Style', 'PushButton', 'String', 'Stop', 'tag', 'but:stop', 'Position', [button.left button.bottom button.width button.height], 'Callback', {@cb_stop, h2, modelname})
    button.left = button.left + button.hnext;
    uicontrol(h1, 'Style', 'PushButton', 'String', 'Step', 'tag', 'but:step', 'Position', [button.left button.bottom button.width button.height], 'Callback', {@cb_step, modelname, stepsize})

    % Draw ParT list
    panParTPos = [0 0 figPos(3) figPos(4)-panTopPos(4)];
    h2 = uipanel('Parent', h, 'tag', 'pan:parT', 'Units', 'pixels', 'Position', panParTPos);

    txtParTPos = [10 panParTPos(4)-30 30 20];
    edtParTPos = [txtParTPos(1)+txtParTPos(3)+5 txtParTPos(2) 40 txtParTPos(4)];

    txt.left = 10;
    txt.bottom = panParTPos(4)-30;
    txt.width = 120;
    txt.height = 15;
    edt.left = txt.left + txt.width + 10;
    edt.bottom = txt.bottom;
    edt.width = 50;
    edt.height = txt.height;
    txt.hnext = txt.height + 5;

    try
        % Set up text and edit boxes for the ParT struct
        names = fieldnames(ParT);
        for i = 1:length(names)
            name = char(names(i));
            subnames = fieldnames(ParT.(name));
            for j = 1:length(subnames)
                subname = char(subnames(j));
                uicontrol(h2, 'Style', 'text', 'Position', [txt.left txt.bottom txt.width txt.height], 'String', [name ' ' subname], 'BackgroundColor', 'white');
                uicontrol(h2, 'Style', 'edit', 'Position', [edt.left edt.bottom edt.width edt.height], 'String', ParT.(name).(subname), 'Callback', {@cb_setParT, name, subname, modelname});
                txt.bottom = txt.bottom - txt.hnext;
                edt.bottom = edt.bottom - txt.hnext;
            end
        end
    catch
        disp('Catch block: ParT struct probably not declared.');
    end
%     uicontrol(h2, 'Style', 'text', 'Position', [txt.left txt.bottom txt.width txt.height], 'String', 'Test q2', 'BackgroundColor', 'white');
%     uicontrol(h2, 'Tag', 'test', 'Style', 'text', 'Position', [edt.left edt.bottom edt.width edt.height], 'String', 'Out_CRANE_Joint2Angle');
%     txt.bottom = txt.bottom - txt.hnext;
%     edt.bottom = edt.bottom - txt.hnext;
end

%%  Callbacks for MYGUI
function cb_step(hObject,eventdata, modelname, stepsize)
    set_param(modelname, 'SimulationCommand', 'step');
end

function cb_startpause(hObject,eventdata,modelname, stepsize)
    global Enable_ACT ParF_ACT2_initPupper ParF_ACT2_initPlower ParF_ACT3_initPupper ParF_ACT3_initPlower;
    global simstatus h_timer;
    persistent var;
    simstatus
    if isempty(var)
        disp('var is empty')
        var = 1;
        simstatus = 'initialise';
    end
    disp('hei')
    simstatus
   switch simstatus
       case {'initialise', 'standby', 'stopped'}
           if strcmp(simstatus, 'initialise') && (strcmp(modelname, 'Crane_test')||strcmp(modelname, 'Crane_Sim')||strcmp(modelname, 'CyberSea_Crane_Simulator')||strcmp(modelname, 'Hydraulics')||strcmp(modelname, 'CraneWinchLoad'))
           % Check if there are hydraulics in the model
               % Set Enable_ACT = 0;
               Enable_ACT = 0
                              
               % Run one step of the model
               set_param(modelname, 'SimulationCommand', 'start');
               set_param(modelname, 'SimulationCommand', 'pause');
               
               % Get tau_2 and tau_3
               obj_tau = get_param([modelname '/Plotting/SumT'], 'RuntimeObject');
               tau = obj_tau.InputPort(1).Data;
%                size(tau)
               tau = tau(2:3)
               for i = 1:5
                   obj_tau = get_param([modelname '/Plotting/SumT'], 'RuntimeObject');
                   tau = obj_tau.InputPort(1).Data;
                   tau = tau(2:3)
                   if abs(tau(1)) == 0 || abs(tau(2)) == 0
                       set_param(modelname, 'SimulationCommand', 'step');
                   else
                       i
                       break;
                   end
                       
               end
               set_param(modelname, 'SimulationCommand', 'stop');
               
               % Compute F_2 fom tau_2 and F3 from tau_3
               Crane;
               c = [ParF_ACT2_LinkCylinderAdditionalAngle ParF_ACT3_LinkCylinderAdditionalAngle]' +ParF_CRANE_DHparameters(4,2:3)';
               K = [ParF_ACT2_K; ParF_ACT3_K];
               L = [ParF_ACT2_L; ParF_ACT3_L];
               P = sqrt(K.^2+L.^2-2*K.*L.*cos(c));
               F = P.*tau./(L.*K.*sin(c));

               % Compute p10 and p20
               A21 = pi*(.5*ParF_ACT2_CylinderDiameter)^2;
               A22 = .25*pi*(ParF_ACT2_CylinderDiameter^2-ParF_ACT2_RodDiameter^2);
               A31 = pi*(.5*ParF_ACT3_CylinderDiameter)^2;
               A32 = .25*pi*(ParF_ACT3_CylinderDiameter^2-ParF_ACT3_RodDiameter^2);
               A1 = [A21; A31];
               A2 = [A22; A32];
               P2 = (A1*ParF_HPU_SupplyPressure - F)./(A1+A2);
               P1 = ParF_HPU_SupplyPressure - P2;
               % Set p10 and p20 in the model
               ParF_ACT2_initPupper = P2(1)
               ParF_ACT2_initPlower = P1(1)
               ParF_ACT3_initPupper = P2(2)
               ParF_ACT3_initPlower = P1(2)
               
               % Set Enable_ACT = 1;
               Enable_ACT = 1;
               
           end % of initialisation
           
           
           % Start simulating, i.e. set simstatus
           simstatus = 'running';
           try
               start(t); % should be h_timer?
           catch
               h_timer = timer('TimerFcn', {@cb_timerfcn, modelname, stepsize},'Period', .5, 'ExecutionMode', 'fixedRate');
               start(h_timer);
               disp('timer(t)');
           end
           set(hObject, 'String', 'Pause');
           set_param(modelname, 'SimulationCommand', 'start');
           set_param(modelname, 'SimulationCommand', 'pause');
%            get_param(modelname, 'SimulationStatus')
       case 'running'
           simstatus = 'paused';
           set(hObject, 'String', 'Resume');
       case 'paused'
           simstatus = 'running';
           set(hObject, 'String', 'Pause');
       otherwise
           disp('Unknown SimulationStatus in cb_startpause');
   end

end

function cb_stop(hObject,eventdata,startpausebutton,modelname)
   global simstatus h_timer;

   simstatus = 'stopped';
   try
       stop(h_timer);
       delete(h_timer);
   end
   set_param(modelname, 'SimulationCommand', 'stop');
   set(startpausebutton, 'String', 'Start');
end


function cb_setParT(hObject, eventdata, name, subname, modelname)
    global ParT;
    strval = get(hObject, 'String');
    if isnumeric(str2double(strval))
        ParT.(name).(subname) = str2double(strval);
        set_param(modelname, 'SimulationCommand', 'update');
    end
end

function cb_timerfcn(hObject, eventdata, modelname, stepsize)
    global simstatus;

    switch simstatus
       case 'running'
           time = 0;
           while time < .5
              set_param(modelname, 'SimulationCommand', 'step');
              time = time + str2double(stepsize);
           end
           % Update GUI. Make a GUI utility function.
           if strcmp(modelname, 'Crane_test') || strcmp(modelname, 'Crane_Sim')% || strcmp(modelname, 'CraneWinchLoad')
%                disp('Updating gui');
               updatecranefig(modelname);
           end
       case 'paused'
           
        case 'stopped'
%            set_param(modelname, 'SimulationCommand', 'stop');
%            stop(t);
            disp('cb_timerfcn: simstatus = stopped. Something is wrong..')
      otherwise
           fprintf('Unknown SimulationStatus in cb_timerfcn: %s', simstatus);
    end
end

%%  Utility functions for MYGUI
function updatecranefig(modelname)
    persistent haxes_xz; %haxes_xy obj_q2 obj_q3
%     disp('enter updatecraneconfig');
    L1 = 10;
    L2 = 20;
    L3 = 7;

    if ~strcmp('stopped', get_param(modelname, 'SimulationStatus'))
%             obj_q1 = get_param([modelname '/Plotting/J1 angle'], 'RuntimeObject');
            obj_q2 = get_param([modelname '/Plotting/J2 angle'], 'RuntimeObject');
            obj_q3 = get_param([modelname '/Plotting/J3 angle'], 'RuntimeObject');
%             obj_boomradius = get_param([modelname '/Plotting/Boom radius'], 'RuntimeObject');
% disp('in updagecranefig1')
%             q1 = obj_q1.InputPort(1).Data
            q2 = obj_q2.InputPort(1).Data;
            q3 = obj_q3.InputPort(1).Data;
%             boomradius = obj_boomradius.InputPort(1).Data;
            
            x = [0 0 L2*cosd(q2) L2*cosd(q2)+L3*cosd(q2+q3)];
            z = [0 L1 L1+L2*sind(q2) L1+L2*sind(q2)+L3*sind(q2+q3)];
%             radius = 10;
            
            % Create xz axes if it doesn't exists
            xlim = [-2 30];
            ylim = [0 30];
%             xlim = [-10 10];
%             ylim = [-20 10];
            if ishandle(haxes_xz)
                plot(haxes_xz, x,z);
                set(haxes_xz, 'XLim', xlim, 'YLim', ylim);
            else
                haxes_xz = axes('Position', [.5 .5 .3 .3]); 
                plot(haxes_xz,x,z);
                set(haxes_xz, 'XLim', xlim, 'YLim', ylim);
            end
    end
end
