function InertiaTensor = ComputeInertiaTensor(LinkDimensions,Mass, dk, db)

numberOfLinks = size(LinkDimensions,1);

% dk - king thickness
% db - box thickness
rho = 7800;


InertiaTensor = zeros(3,3,numberOfLinks);
for i = 1:numberOfLinks
    switch LinkDimensions(i,4)
        case 1 % Cylinder
            % The matrix is transposed because Matlab stores it in column major order in memory
            ro = LinkDimensions(i,2);
            ri = ro-dk;
            Ixy = .25*Mass(i)*(ro^2+ri^2) + 1/12*Mass(i)*LinkDimensions(i,2)^2;
            InertiaTensor(:,:,i) = diag([Ixy, Ixy, .5*Mass(i)*(ro^2+ri^2)])';
        case 2 % Rectangular block
            % The matrix is transposed because Matlab stores it in column major order in memory
            L = LinkDimensions(i,1);
            H = LinkDimensions(i,2);
            B = LinkDimensions(i,3);
            mi = rho*L*(B-2*db)*(H-2*db);
            Ix = 1/12*(B^2+H^2)*Mass(i)+1/3*mi*db*(B+H-db);
            Iy = 1/12*(B^2+H^2)*Mass(i)+1/3*mi*db*(B-db);
            Iz = 1/12*(L^2+H^2)*Mass(i)+1/3*mi*db*(H-db);
            InertiaTensor(:,:,i) = diag([Ix, Iy, Iz])';
    end
end

end