%% System wide parameters
ParC_SYSTEM_GravitationalConstant = [0 0 9.81];



% %% Configuration of vessel
% ParF_VESSEL_InitialPosition = [0 0 0 0 0 0];
% load('RAO_freq');
% ParF_VESSEL_RAOfreq = RAO_freq;
% clear RAO_freq;


%% Configuration of crane
ParF_CRANE_BaseFramePosition = [0 0 -12]; % Vector from crane base frame origin to vessel CG [m]
ParF_CRANE_VesselBodyToBaseFrameRotation = [1 0 0; 0 -1 0; 0 0 -1];

kingheight = 3.9;
kingradius = 1.8;
dk = .15; % Thickness of the king

link2_length = 21.1;
link2_height = 2;
link2_width = .75;

link3_length = 12.7;
link3_height = 1.4;
link3_width = .5;
db = .03; % Plate thickness in the box booms

ParF_CRANE_LinkDimensions = [kingheight, kingradius, 0, 1;
                            link2_length, link2_height, link2_width, 2;
                            link3_length, link3_height, link3_width, 2]; % Cylinder: Height [m], radius [m], not in use, type=1
                                                                     % Rectangular block: Length [m], height [m], width [m], type=2

ParF_CRANE_DHparameters = [0, pi/2, kingheight, 0*pi/180;
                          link2_length, 0, 0, 50*pi/180;
                          link3_length, 0, 0, -140*pi/180]'; % [m] and [rad/s]. The matrix is transposed because Matlab stores it in column major order in memory

ParF_CRANE_NumberOfLinks = 3;
ParF_CRANE_TypeOfJoint = [0,0,0]; % 0=revolute, 1=prismatic
ri = kingradius - dk; % Inner king radius
ParF_CRANE_Mass = [pi*(kingradius^2-ri^2)*kingheight*7800, ...
                   2*7800*(link2_height+link2_width-2*db)*db*link2_length, ...
                   2*7800*(link3_height+link3_width-2*db)*db*link3_length]; % [kg] assuming 20 cm thick king, and 3 cm thick booms


ParF_CRANE_InertiaTensor = ComputeInertiaTensor(ParF_CRANE_LinkDimensions,ParF_CRANE_Mass, dk, db); 
% clear dk db ri;
ParF_CRANE_JointToJointVectors = ComputeJointToJointVectors(ParF_CRANE_DHparameters);
ParF_CRANE_JointToCGVectors = [[0, ParF_CRANE_JointToJointVectors(2,1)/2, 0]',...
                               [ParF_CRANE_JointToJointVectors(1,2)/2,0,0]',...
                               [ParF_CRANE_JointToJointVectors(1,3)/2,0,0]'];

ParF_CRANE_JointCoulombFrictionCoeff = 0*[1,1,1]; % [N]
% ParF_CRANE_JointFreeFallVel = [1,1,1]; % [rad/s]. Used to compute approximate viscous friction coefficients
d = .5*[0, link2_length, link3_length]; % Distance from CM to joint used when determining viscous friction
a_visc = .3; % w(t1) = a*w(0)
t1_visc = 5;
J = [ParF_CRANE_InertiaTensor(3,3,1), ParF_CRANE_InertiaTensor(3,3,2), ParF_CRANE_InertiaTensor(3,3,3)]+ParF_CRANE_Mass.*d.^2; % Parallel axes theorem
ParF_CRANE_JointViscousFrictionCoeff = -J*log(a_visc)/t1_visc;
% ParF_CRANE_JointViscousFrictionCoeff = ParF_CRANE_Mass.*sqrt(sum(ParF_CRANE_JointToCGVectors.^2))./ParF_CRANE_JointFreeFallVel*9.81;
ParF_CRANE_Karnopp = 1; % 0=Karnopp friction model off, 1=Karnopp friction model on
                                                                     

ParF_CRANE_qInit = zeros(ParF_CRANE_NumberOfLinks,1);
for i = 1:ParF_CRANE_NumberOfLinks
    switch ParF_CRANE_TypeOfJoint(i)
        case 0 % Revolute joint
            ParF_CRANE_qInit(i,1) = ParF_CRANE_DHparameters(4,i);
        case 1 % Prismatic joint
            ParF_CRANE_qInit(i,1) = ParF_CRANE_DHparameters(3,i);
    end
end


%% Configuration of actuators

%% HPU
ParF_HPU_SupplyPressure = 300E5; % [Pa]

%% ACT 
ParF_ACT_HydraulicBulkModulus = 7e8;
% ParF_ACT_Retr = 1000;
% ParF_ACT_rho = 900;
% ParF_ACT_nu = 30e-6;
% ParF_ACT_Cd = .6;

%% ACT1
ParF_ACT1_Index = 1;
% ParF_ACT1_PID = [0,15,15]; % PID parameters


%% ACT2
ParF_ACT2_Index = 2;
K1 = kingheight;
K2 = kingradius;
L1 = 8.7;
L2 = 2.3;
offset = pi/2;
ParF_ACT2_K = sqrt(K1^2+K2^2);
ParF_ACT2_L = sqrt(L1^2+L2^2);
k = tan(K2/K1);
l = tan(L2/L1);

% c = offset-k-l;
% lengdaer = sqrt(ParF_ACT2_K^2 + ParF_ACT2_L^2 - 2*ParF_ACT2_K*ParF_ACT2_L*cos(c))
% tau2 = force2torque(K1,K2,L1,L2,offset,[0:.1:72]*pi/180);

ParF_ACT2_LinkCylinderGeometry = [ParF_ACT2_K,ParF_ACT2_L]; % [m]
ParF_ACT2_LinkCylinderAdditionalAngle = offset-k-l; % [rad]
ParF_ACT2_JointLimits = [0,70]; %[-270 80]; % Angle limits
ParF_ACT2_CylinderBufferLength = .3;
[RodLength, CylinderStroke, Pmax2, Pmin2] = compCylinder(ParF_ACT2_LinkCylinderGeometry, ParF_ACT2_JointLimits+180/pi*ParF_ACT2_LinkCylinderAdditionalAngle,ParF_ACT2_CylinderBufferLength);
ParF_ACT2_CylinderStroke = CylinderStroke; % [m]
ParF_ACT2_CylinderRodLength = RodLength; % [m]
% F0 = Fcyl(ParF_ACT2_LinkCylinderGeometry, link2_length, 1e5, 90, 0);
F0 = 1000e3*9.81; %2.6e6;
ParF_ACT2_CylinderDiameter = 2*sqrt(F0/(pi*ParF_HPU_SupplyPressure));
Ap2 = .25*ParF_ACT2_CylinderDiameter^2*pi;
ParF_ACT2_RodDiameter = sqrt(.2)*ParF_ACT2_CylinderDiameter;
% ParF_ACT2_V0 = [0, pi*(ParF_ACT2_CylinderDiameter^2 - ParF_ACT2_RodDiameter)*ParF_ACT2_CylinderStroke]; % [Vlower, Vupper]
ParF_ACT2_PistonRelativeDamping = .5;
ParF_ACT2_PistonMass = .25^2*pi*ParF_ACT2_RodDiameter^2*ParF_ACT2_CylinderRodLength*7800; % .25*Volume*rho_steel

vp2_max = .1;
ParF_ACT2_MaxPistonVelocity = vp2_max;
ParF_ACT2_ValveMaxFlow = vp2_max*pi*(ParF_ACT2_CylinderDiameter/2)^2; % Design speed X piston area
% ParF_ACT2_ValveMaxSpoolPosition = .0005; % metres
ParF_ACT2_ValveDeadzone = 0;


a = ParF_ACT2_LinkCylinderGeometry(1);
b = ParF_ACT2_LinkCylinderGeometry(2);
ParF_ACT2_PistonPositionInit = sqrt(a^2+b^2-2*a*b*cos(ParF_ACT2_LinkCylinderAdditionalAngle+ParF_CRANE_qInit(2,1)))...
    -ParF_ACT2_CylinderRodLength;

b_tilde2 = .25*pi*ParF_ACT2_CylinderDiameter^2*vp2_max*sqrt(3*ParF_ACT_HydraulicBulkModulus/ParF_HPU_SupplyPressure)/.6;

clear K1 K2 L1 L2 offset K L k l F0 a b

%% ACT3
ParF_ACT3_Index = 3;
K1 = 8.1;
K2 = 2.3;
L1 = 2.7;
L2 = 0;
offset = pi;
ParF_ACT3_K = sqrt(K1^2+K2^2);
ParF_ACT3_L = sqrt(L1^2+L2^2);
k = tan(K2/K1);
l = tan(L2/L1);

% tau3 = force2torque(K1,K2,L1,L2,offset,[-150:.1:-20]*pi/180);

ParF_ACT3_LinkCylinderGeometry = [ParF_ACT3_K,ParF_ACT3_L]; % [m]
ParF_ACT3_LinkCylinderAdditionalAngle = offset-k-l; % [rad]
ParF_ACT3_JointLimits = [-150, -20]; % [-400 400];
ParF_ACT3_CylinderBufferLength = .2;
[RodLength, CylinderStroke, Pmax3, Pmin3] = compCylinder(ParF_ACT3_LinkCylinderGeometry, ParF_ACT3_JointLimits+180/pi*ParF_ACT3_LinkCylinderAdditionalAngle,ParF_ACT3_CylinderBufferLength);
ParF_ACT3_CylinderStroke = CylinderStroke; % [m]
ParF_ACT3_CylinderRodLength = RodLength; % [m]
% F0 = Fcyl(ParF_ACT3_LinkCylinderGeometry, link3_length, 1e5, 135, 90);
F0 = 400e3*9.81; % 1.9e5;
ParF_ACT3_CylinderDiameter = 2*sqrt(F0/(pi*ParF_HPU_SupplyPressure));
ParF_ACT3_RodDiameter = sqrt(.2)*ParF_ACT3_CylinderDiameter;
ParF_ACT3_PistonRelativeDamping = .5;
ParF_ACT3_PistonMass = .25^2*pi*ParF_ACT3_RodDiameter^2*ParF_ACT3_CylinderRodLength*7800; % .25*Volume*rho_steel

vp3_max = .1;
ParF_ACT3_MaxPistonVelocity = vp3_max;
ParF_ACT3_ValveMaxFlow = vp3_max*pi*(ParF_ACT3_CylinderDiameter/2)^2; % Design speed X piston area
% ParF_ACT3_ValveMaxSpoolPosition = .0005; % metres

a = ParF_ACT3_LinkCylinderGeometry(1);
b = ParF_ACT3_LinkCylinderGeometry(2);
ParF_ACT3_PistonPositionInit = sqrt(a^2+b^2-2*a*b*cos(ParF_ACT3_LinkCylinderAdditionalAngle+ParF_CRANE_qInit(3,1)))...
    -ParF_ACT3_CylinderRodLength;

b_tilde3 = .25*pi*ParF_ACT3_CylinderDiameter^2*vp3_max*sqrt(3*ParF_ACT_HydraulicBulkModulus/ParF_HPU_SupplyPressure)/.6;

clear kingheight kingradius link2_length link2_height link2_width link3_length link3_height link3_width RodLength CylinderStroke;
clear K1 K2 L1 L2 offset K L k l F0 a b

%% Increass link masses and inertias due to rod masses
% ParF_CRANE_Mass = ParF_CRANE_Mass + [ParF_ACT2_PistonMass ParF_ACT2_PistonMass 0]*eye(3);
% ParF_CRANE_InertiaTensor = ComputeInertiaTensor(ParF_CRANE_LinkDimensions,ParF_CRANE_Mass, dk, db); 
% J = [ParF_CRANE_InertiaTensor(3,3,1), ParF_CRANE_InertiaTensor(3,3,2), ParF_CRANE_InertiaTensor(3,3,3)]+ParF_CRANE_Mass.*d.^2; % Parallel axes theorem
% ParF_CRANE_JointViscousFrictionCoeff = -J*log(a_visc)/t1_visc;

%% Parameters for testing hydraulics
% FL = 20e3*9.81;
% A1 = .25*pi*ParF_ACT2_CylinderDiameter^2;
% A2 = A1-.25*pi*ParF_ACT2_RodDiameter^2;
% initP2 = (A1*ParF_HPU_SupplyPressure - FL)/(A1+A2)
% initP1 = ParF_HPU_SupplyPressure - initP2

%% Parameters set directly in the mask for each ACTi
% Initial pressure upper volume
% Initial pressure lower volume
% Offset BVA
% Offset BVB
% Percentage of max leakage


global Enable_ACT ParF_ACT2_initPupper ParF_ACT2_initPlower ParF_ACT3_initPupper ParF_ACT3_initPlower;
Enable_ACT = 1;
ParF_ACT2_initPupper = 1e5;
ParF_ACT2_initPlower = 1e5;
ParF_ACT3_initPupper = 1e5;
ParF_ACT3_initPlower = 1e5;
