%clear DH_matrix, q_dot_falling, Mass, LinkDimensions;

numberOfLinks = 3;
typeOfJoint = [0,0,0];
DH_matrix = [0, pi/2, 50, 0*pi/180; 20, 0, 0, -10*pi/180; 20, 0, 0, -160*pi/180];
g = 9.81;

RotMtrx = zeros(3,3,numberOfLinks);
for i = 1:numberOfLinks
    RotMtrx(:,:,i) = ComputeRotationMatrix(DH_matrix(i,2),DH_matrix(i,4))'; % The matrix is transposed because Matlab stores it in column major order in memory
end

q_dot_falling = [1,1,1]; % Could be used to compute approximate viscous friction coefficient
Mass = [30000, 10000, 10000]; % Mass of each link
LinkDimensions = [50, 3, 0, 1; 20, 1, 1, 2; 20, 1, 1, 2]; % Cylinder: Height, radius, not in use, type=1
                                                     % Rectangular block: Length, height, width, type=2


% ParF
%ParF_CRANE_KingHeight = KingHeight;
%ParF_CRANE_KingRadius = KingRadius;
ParF_CRANE_BaseFramePosition = [0, 0, 0]';
ParF_CRANE_VesselBodyToBaseFrameRotation = [1 0 0; 0 -1 0; 0 0 -1];
ParF_CRANE_DHmatrix = DH_matrix'; % The matrix is transposed because Matlab stores it in column major order in memory
ParF_CRANE_RotMtrx = RotMtrx;
ParF_CRANE_NumberOfLinks = numberOfLinks;
ParF_CRANE_TypeOfJoint = typeOfJoint;
ParF_CRANE_Mass = Mass;
ParF_CRANE_LinkDimensions = LinkDimensions;

ParF_CRANE_InertiaTensor = zeros(3,3,ParF_CRANE_NumberOfLinks);
for i = 1:ParF_CRANE_NumberOfLinks
    switch LinkDimensions(i,4)
        case 1
            % The matrix is transposed because Matlab stores it in column major order in memory
            ParF_CRANE_InertiaTensor(:,:,i) = diag([1/12*Mass(i)*LinkDimensions(i,1)^2 + 1/4*Mass(i)*LinkDimensions(i,2)^2,...
                1/12*Mass(i)*LinkDimensions(i,1)^2 + 1/4*Mass(i)*LinkDimensions(i,2)^2,...
                1/2*Mass(i)*LinkDimensions(i,2)^2])';
        case 2
            % The matrix is transposed because Matlab stores it in column major order in memory
            ParF_CRANE_InertiaTensor(:,:,i) = diag([1/12*Mass(i)*(LinkDimensions(i,2)^2+LinkDimensions(i,3)^2),...
                                 1/12*Mass(i)*(LinkDimensions(i,1)^2+LinkDimensions(i,3)^2),...
                                 1/12*Mass(i)*(LinkDimensions(i,1)^2+LinkDimensions(i,2)^2)])';
    end
end

ParF_CRANE_JointToJointVectors = zeros(3,numberOfLinks);
%ParF_CRANE_JointToJointVectors = [0, 50, 0; 20, 0, 0; 20, 0, 0]';
for i = 1:numberOfLinks
    ParF_CRANE_JointToJointVectors(:,i) = ComputeJointToJointVector(DH_matrix(i,:));
end
% TODO: Create automatic assignment of the following vectors

% ******* THIS PART NEEDS VERIFICATION ************
ParF_CRANE_JointToCGVectors = [[0, LinkDimensions(1,1)/2, 0]',...
                              [ParF_CRANE_JointToJointVectors(1,2)/2,0,0]',...
                              [ParF_CRANE_JointToJointVectors(1,3)/2,0,0]'];
% ******* END VERIFICATION PART *******************                          
                          
ParF_CRANE_JointCoulombFrictionCoeff = [1e3,1e3,1e3]'; % Approximate value
for i = 1:numberOfLinks
    ParF_CRANE_JointViscousFrictionCoeff(i) = Mass(i)*norm(ParF_CRANE_JointToCGVectors(:,i))*g/q_dot_falling(i);
    %ParF_CRANE_JointViscousFrictionCoeff(i) = 0; %Just for testing
end
%ParF_CRANE_JointViscousFrictionCoeff(3) = 0; %Just for testing
%ParF_CRANE_JointViscousFrictionCoeff(2) = 1e7; %Just for testing

ParF_CRANE_qInit = zeros(ParF_CRANE_NumberOfLinks,1);
for i = 1:ParF_CRANE_NumberOfLinks
    switch ParF_CRANE_TypeOfJoint(i)
        case 0
            ParF_CRANE_qInit(i,1) = DH_matrix(i,4);
        case 1
            ParF_CRANE_qInit(i,1) = DH_matrix(i,3);
    end
end

ParF_ACT_LinkCylinderGeometry = [0,0;4,8;2,8];
ParF_ACT_LinkCylinderAdditionalAngle = [0, pi/2, pi];


% ParF for the actuators

ParF_ACT_CylinderRodLength = [0,6,6]';
ParF_ACT_CylinderStroke = [-1,6,6]';
ParF_ACT_SupplyPressure = 350e5;
%ParF_ACT_PistonCoulombFrictionCoeff = 1e3;


% ****** This evaluation of the initial piston position should possibly be implemented inside the Simulink model *******
ParF_ACT_PistonPositionInit = zeros(numberOfLinks,1);
for i = 1:numberOfLinks
    if ~(ParF_ACT_CylinderStroke(i,1)==-1)
        a = ParF_ACT_LinkCylinderGeometry(i,1);
        b = ParF_ACT_LinkCylinderGeometry(i,2);
        ParF_ACT_PistonPositionInit(i,1) = sqrt(a^2+b^2-2*a*b*cos(ParF_ACT_LinkCylinderAdditionalAngle(i)+ParF_CRANE_qInit(i,1)))...
              -ParF_ACT_CylinderRodLength(i,1);
    end
end