function [C_a, C_s, V_sub, A_l, V_l, b1, b2] = load_geometry(geom, H_l, L_l, B_l)

switch geom
    case 'box'
        % Projected area and volume of the box
        
        % Reduction factor used in Andreas' project to make the box
        % neutrally buoyant
        k = .468;

        V_l = k*L_l*B_l*H_l;
        A_l = B_l*L_l;        
        
        % Nondimensional added mass
        C_a = [0 1]/k;
        z_add = [0 1];
                
        % Nondimensional slamming
        C_s = [0 2*L_l*B_l/H_l 2*L_l*B_l/H_l 0];
        z_slam = [0 1e-6 1-1e-6 1];

        % Submerged volume
        z_sub = [0 1];
        V_sub = [0 V_l];
        
        % Damping terms
        C_D = 1.2;
        b1 = 0; % Nondimensional linear damping
        b2 = C_D; % Nondimensional quadratic damping. From Skaare (and White99): C_D = 0.47

    case 'sphere'
        % Projected area and volume of the sphere
        A_l = .25*pi*H_l^2;
        V_l = pi*H_l^3/6;
        
        % Added mass term
        a1 = .5;
        z_a1 = 0:.05:.3733;
        add1 = a1.*z_a1.^2;
        
        a2 = -.5362;
        b2 = 1.2187;
        c2 = -.3924;
        z_a2 = .75:.05:1.123;
        add2 = a2.*z_a2.^2 + b2.*z_a2 + c2;
        
        z_add = [z_a1 z_a2];
        C_a = [add1 add2];
        
        % Slamming term
        z1 = 0;
        z2 = .3733;
        z3 = .7467;
        z4 = 1.1227;
        z_slam = [z1 z2 z3 z4];
        
        C_s1 = 0;
        C_s2 = 1.9;
        C_s3 = C_s2;
        C_s4 = 0;
        C_s = [C_s1 C_s2 C_s3 C_s4];
        
        % Submerged volume
        z_sub = 0:.05/H_l:1;
        V_sub = .5*pi*H_l^3.*(1- 2*z_sub/3).*z_sub.^2;
        
        % Damping terms
        C_D = .47;
        b1 = 0; % Nondimensional linear damping
        b2 = C_D; % Nondimensional quadratic damping. From Skaare (and White99): C_D = 0.47

    otherwise
        disp('error in the cable function: wrong input')
        disp('Entering debug mode')
        keyboard;
end

C_a = [C_a; z_add];
C_s = [C_s; z_slam];
V_sub = [V_sub; z_sub];